/**
 * 共用 loader 或 plugin 配置参数
 */

const path = require('path')
const fs = require('fs')
const { getMode, getSourceMap, getProjectRoot } = require('../common/util')

module.exports = () => {
  const mode = getMode()
  const sourceMap = !!getSourceMap()
  const projectRoot = getProjectRoot()

  const isExist2browserslistrc = fs.existsSync(path.resolve(projectRoot, '.browserslistrc'))

  const babelPresetEnvConfig = {
    targets: { 'browsers': ['android >= 4'] }
  }
  const postcssPresetEnvConfig = {
    stage: 0,
    browsers: ['> 0.01%']
  }

  if (isExist2browserslistrc) {
    delete babelPresetEnvConfig.targets
    delete postcssPresetEnvConfig.browsers
  }

  const babelLoaderOptions = {
    cacheDirectory: true,
    presets: [
      [
        '@babel/preset-env', babelPresetEnvConfig
      ]
    ],
    plugins: [
      ['@babel/plugin-proposal-class-properties', { loose: true }],
      '@babel/plugin-proposal-nullish-coalescing-operator',
      '@babel/plugin-proposal-optional-chaining',
      '@babel/plugin-syntax-dynamic-import',
      ['@babel/plugin-transform-runtime', { corejs: 2 }]
    ]
  }

  const cssLoaderOptions = { sourceMap }

  const postcssLoaderOptions = {
    sourceMap,
    plugins: () => [
      require('postcss-preset-env')(postcssPresetEnvConfig)
    ]
  }

  const sassLoaderOptions = { sourceMap }

  const globalSassResources = path.resolve(projectRoot, 'src/style/var.scss')
  const sassResourcesOptions = fs.existsSync(globalSassResources) ? { resources: [ globalSassResources ] } : void 0

  const miniCssOptions = {
    hmr: mode === 'development',
    reloadAll: true,
    publicPath: '../'
  }

  return {
    babelLoaderOptions,
    cssLoaderOptions,
    postcssLoaderOptions,
    sassLoaderOptions,
    sassResourcesOptions,
    miniCssOptions
  }
}
